# -*- coding: utf-8 -*-

"""Calculates the current version number.

If possible, uses output of “git describe” modified to conform to the
visioning scheme that setuptools uses (see PEP 386).  Releases must be
labelled with annotated tags (signed tags are annotated) of the following
format:

   v<num>(.<num>)+ [ {a|b|c|rc} <num> (.<num>)* ]

If “git describe” returns an error (likely because we're in an unpacked copy
of a release tarball, rather than a git working copy), or returns a tag that
does not match the above format, version is read from .version file.

To use this script, simply import it in your setup.py file, and use the results
of getVersion() as your package version:

    import version
    setup(
        version=version.getVersion(),
        .
        .
        .
    )

This will automatically update the .version file.  The .version file should
*not* be checked into git but it *should* be included in sdist tarballs (as
should version.py file).  To do this, run:

    echo include .version version.py >>MANIFEST.in
    echo .version >>.gitignore

With that setup, a new release can be labelled by simply invoking:

    git tag -s v1.0
"""

__author__ = ('Douglas Creager <dcreager@dcreager.net>',
              'Michal Nazarewicz <mina86@mina86.com>')
__license__ = 'This file is placed into the public domain.'
__maintainer__ = 'Michal Nazarewicz'
__email__ = 'mina86@mina86.com'

__all__ = ('getVersion',)


import re
import subprocess
import sys


RELEASE_VERSION_FILE = '.version'

# http://www.python.org/dev/peps/pep-0386/
_PEP386_SHORT_VERSION_RE = r'\d+(?:\.\d+)+(?:(?:[abc]|rc)\d+(?:\.\d+)*)?'
_PEP386_VERSION_RE = r'^%s(?:\.post\d+)?(?:\.dev\d+)?$' % (
    _PEP386_SHORT_VERSION_RE)
_GIT_DESCRIPTION_RE = r'^v(?P<ver>%s)-(?P<commits>\d+)-g(?P<sha>[\da-f]+)$' % (
    _PEP386_SHORT_VERSION_RE)


def readGitVersion():
    try:
        proc = subprocess.Popen(('git', 'describe', '--long',
                                 '--match', 'v[0-9]*.*'),
                                stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        data, _ = proc.communicate()
        if proc.returncode:
            return None
        ver = data.splitlines()[0].strip()
    except:
        return None

    if not ver:
        return None
    m = re.search(_GIT_DESCRIPTION_RE, ver)
    if not m:
        sys.stderr.write('version: git description (%s) is invalid, '
                         'ignoring\n' % ver)
        return None

    commits = int(m.group('commits'))
    if not commits:
        return m.group('ver')
    else:
        return '%s.post%d.dev%d' % (
            m.group('ver'), commits, int(m.group('sha'), 16))


def readReleaseVersion(warn):
    try:
        fd = open(RELEASE_VERSION_FILE)
        try:
            ver = fd.readline().strip()
        finally:
            fd.close()
        if warn and not re.search(_PEP386_VERSION_RE, ver):
            sys.stderr.write('version: release version (%s) is invalid, '
                             'will use it anyway\n' % ver)
        return ver
    except:
        return None


def writeReleaseVersion(version):
    fd = open(RELEASE_VERSION_FILE, 'w')
    fd.write('%s\n' % version)
    fd.close()


def getVersion():
    version = readGitVersion()
    release_version = readReleaseVersion(not version)
    version = version or release_version
    if not version:
        raise ValueError('Cannot find the version number')
    if version != release_version:
        writeReleaseVersion(version)
    return version


if __name__ == '__main__':
    print getVersion()



#!/usr/bin/python

"""trie module example code."""

__author__     = 'Michal Nazarewicz <mina86@mina86.com>'
__copyright__  = 'Copyright 2014 Google Inc.'


import os
import stat
import sys

import pygtrie


print 'Storing file information in the trie'
print '===================================='
print

ROOT_DIR = '/usr/local'
SUB_DIR = os.path.join(ROOT_DIR, 'lib')
SUB_DIRS = tuple(os.path.join(ROOT_DIR, d)
                 for d in ('lib', 'lib32', 'lib64', 'share'))

t = pygtrie.StringTrie(separator=os.path.sep)

# Read sizes regular files into a Trie
for dirpath, unused_dirnames, filenames in os.walk(ROOT_DIR):
    for filename in filenames:
        filename = os.path.join(dirpath, filename)
        try:
            filestat = os.stat(filename)
        except OSError:
            continue
        if stat.S_IFMT(filestat.st_mode) == stat.S_IFREG:
            t[filename] = filestat.st_size

# Size of all files we've scanned
print 'Size of %s: %d' % (ROOT_DIR, sum(t.itervalues()))

# Size of all files of a sub-directory
print 'Size of %s: %d' % (SUB_DIR, sum(t.itervalues(prefix=SUB_DIR)))

# Check existence of some directories
for directory in SUB_DIRS:
    print directory, 'exists' if t.has_subtrie(directory) else 'does not exist'

# Drop a subtrie
print 'Dropping', SUB_DIR
del t[SUB_DIR:]
print 'Size of %s: %d' % (ROOT_DIR, sum(t.itervalues()))
for directory in SUB_DIRS:
    print directory, 'exists' if t.has_subtrie(directory) else 'does not exist'


print
print 'Storing URL handlers map'
print '========================'
print

t = pygtrie.CharTrie()
t['/'] = lambda url: sys.stdout.write('Root handler: %s\n' % url)
t['/foo'] = lambda url: sys.stdout.write('Foo handler: %s\n' % url)
t['/foobar'] = lambda url: sys.stdout.write('FooBar handler: %s\n' % url)
t['/baz'] = lambda url: sys.stdout.write('Baz handler: %s\n' % url)

for url in ('/', '/foo', '/foot', '/foobar', 'invalid', '/foobarbaz', '/ba'):
    key, handler = t.longest_prefix(url)
    if key is not None:
        handler(url)
    else:
        print 'Unable to handle', repr(url)


if not os.isatty(0):
    sys.exit(0)


try:
    import termios
    import tty

    def getch():
        attr = termios.tcgetattr(0)
        try:
            tty.setraw(0)
            return sys.stdin.read(1)
        finally:
            termios.tcsetattr(0, termios.TCSADRAIN, attr)

except ImportError:
    try:
        from msvcrt import getch
    except ImportError:
        sys.exit(0)


print
print 'Prefix set'
print '=========='
print

ps = pygtrie.PrefixSet(factory=pygtrie.StringTrie)

ps.add('/etc/rc.d')
ps.add('/usr/local/share')
ps.add('/usr/local/lib')
ps.add('/usr')  # Will handle the above two as well
ps.add('/usr/lib')  # Does not change anything

print 'Path prefixes:', ', '.join(iter(ps))
for path in ('/etc', '/etc/rc.d', '/usr', '/usr/local', '/usr/local/lib'):
    print 'Is', path, 'in the set:', ('yes' if path in ps else 'no')


print
print 'Dictionary test'
print '==============='
print

t = pygtrie.CharTrie()
t['cat'] = True
t['caterpillar'] = True
t['car'] = True
t['bar'] = True
t['exit'] = False

print 'Start typing a word, "exit" to stop'
print '(Other words you might want to try: %s)' % ', '.join(sorted(
    k for k in t if k != 'exit'))
print

text = ''
while True:
    ch = getch()
    if ord(ch) < 32:
        print 'Exiting'
        break

    text += ch
    value = t.get(text)
    if value is False:
        print 'Exiting'
        break
    if value is not None:
        print repr(text), 'is a word'
    if t.has_subtrie(text):
        print repr(text), 'is a prefix of a word'
    else:
        print repr(text), 'is not a prefix, going back to empty string'
        text = ''



from distutils.core import setup
import os
import re
import sys
import version

release = version.getVersion()

if len(sys.argv) == 2 and sys.argv[1] == 'builddoc':
    os.execlp('sphinx-build',
              '-Drelease=' + release,
              '-Dversion=' + '.'.join(release.split('.', 2)[0:2]),
              '.', 'html')

with open('README.rst') as f:
    readme = f.read()
with open('version-history.rst') as f:
    readme += '\n' + f.read()

kwargs = {
    'name': 'pygtrie',
    'version': release,
    'description': 'Trie data structure implementation.',
    'long_description': readme,
    'author': 'Michal Nazarewicz',
    'author_email': 'mina86@mina86.com',
    'url': 'https://github.com/google/pygtrie',
    'py_modules': ['pygtrie'],
    'license': 'Apache-2.0',
    'platforms': 'Platform Independent',
    'keywords': ['trie', 'prefix tree', 'data structure'],
    'classifiers': [
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
}

if re.search(r'(?:\d+\.)*\d+', release):
    kwargs['download_url'] = kwargs['url'] + '/tarball/v' + release

setup(**kwargs)



# -*- coding: utf-8 -*-
"""Implementation of a trie data structure.

`Trie data structure <http://en.wikipedia.org/wiki/Trie>`_, also known as radix
or prefix tree, is a tree associating keys to values where all the descendants
of a node have a common prefix (associated with that node).

The trie module contains :class:`pygtrie.Trie`, :class:`pygtrie.CharTrie` and
:class:`pygtrie.StringTrie` classes each implementing a mutable mapping
interface, i.e. :class:`dict` interface.  As such, in most circumstances,
:class:`pygtrie.Trie` could be used as a drop-in replacement for
a :class:`dict`, but the prefix nature of the data structure is trie’s real
strength.

The module also contains :class:`pygtrie.PrefixSet` class which uses a trie to
store a set of prefixes such that a key is contained in the set if it or its
prefix is stored in the set.

Features
--------

- A full mutable mapping implementation.

- Supports iterating over as well as deleting a subtrie.

- Supports prefix checking as well as shortest and longest prefix
  look-up.

- Extensible for any kind of user-defined keys.

- A PrefixSet supports “all keys starting with given prefix” logic.

- Can store any value including None.

For some simple examples see ``example.py`` file.
"""

__author__ = 'Michal Nazarewicz <mina86@mina86.com>'
__copyright__ = 'Copyright 2014 Google Inc.'


import collections as _collections


class ShortKeyError(KeyError):
  """Raised when given key is a prefix of a longer key."""
  pass


_SENTINEL = object()


class _Node(object):
  """A single node of a trie.

  Stores value associated with the node and dictionary of children.
  """
  __slots__ = ('children', 'value')

  def __init__(self):
    self.children = {}
    self.value = _SENTINEL

  def iterate(self, path, shallow=False):
    """Yields all the nodes with values associated to them in the trie.

    Args:
      path: Path leading to this node.  Used to construct the key when
          returning value of this node and as a prefix for children.
      shallow: Perform a shallow traversal, i.e. do not yield nodes if their
          prefix has been yielded.

    Yields:
      ``(path, value)`` tuples.
    """
    node = self

    if node.value is not _SENTINEL:
      yield path, node.value
      if shallow:
        return

    # Use iterative function with stack on the heap so we don't hit Python's
    # recursion depth limits.
    stack = []
    while True:
      stack.append(iter(sorted(node.children.iteritems())))
      path.append(None)

      while True:
        step, node = next(stack[-1], (_SENTINEL, _SENTINEL))

        # We're done with children, go up
        if node is _SENTINEL:
          stack.pop()
          if not stack:
            return
          path.pop()
          continue

        path[-1] = step
        if node.value is not _SENTINEL:
          yield path, node.value
          if shallow:
            continue

        break

  def traverse(self, node_factory, path_conv, path):
    """Traverses the node and returns another type of node from node_factory.

    Args:
      node_factory: Callable function to construct new nodes.
      path_conv: Callable function to convert node path to a key.
      path: Current path for this node.

    Returns:
      An object constructed by calling node_factory(path_conv, path, children,
      value=...), where children are constructed by node_factory from the
      children of this node. There doesn't need to be 1:1 correspondence between
      original nodes in the trie and constructed nodes (see
      make_test_node_and_compress in test.py).
    """
    def children():
      for step, node in sorted(self.children.iteritems()):
        yield node.traverse(node_factory, path_conv, path + [step])

    args = [path_conv, tuple(path), children()]

    if self.value is not _SENTINEL:
      args.append(self.value)

    return node_factory(*args)

  def __eq__(self, other):
    return self.value == other.value and self.children == other.children

  def __ne__(self, other):
    return not self == other  # pylint: disable=g-comparison-negation

  def __nonzero__(self):
    return bool(self.value is not _SENTINEL or self.children)

  __hash__ = None

  def __getstate__(self):
    if self.value is not _SENTINEL:
      return (self.children, self.value)
    elif self.children:
      return self.children

  def __setstate__(self, state):
    if not isinstance(state, tuple):
      state = (state, _SENTINEL)
    self.children, self.value = state


class Trie(_collections.MutableMapping):
  """A trie implementation with dict interface plus some extensions.

  Keys used with the :class:`pygtrie.Trie` must be iterable, yielding hashable
  objects.  In other words, for a given key, ``dict.fromkeys(key)`` must be
  valid.

  In particular, strings work fine as trie keys, however when getting keys back
  from iterkeys() method for example, instead of strings, tuples of characters
  are produced.  For that reason, :class:`pygtrie.CharTrie` or
  :class:`pygtrie.StringTrie` may be preferred when using :class:`pygtrie.Trie`
  with string keys.
  """

  # pylint: disable=invalid-name

  def __init__(self, *args, **kwargs):
    """Initialises the trie.

    Arguments are interpreted the same way :func:`Trie.update` interprets them.
    """
    self._root = _Node()
    self.update(*args, **kwargs)

  def clear(self):
    """Removes all the values from the trie."""
    self._root = _Node()

  def update(self, *args, **kwargs):
    """Updates stored values.  Works like :func:`dict.update`."""
    if len(args) > 1:
      raise ValueError('update() takes at most one positional argument, '
                       '%d given.' % len(args))
    # We have this here instead of just letting MutableMapping.update() handle
    # things because it will iterate over keys and for each key retrieve the
    # value.  With Trie, this may be expensive since the path to the node
    # would have to be walked twice.  Instead, we have our own implementation
    # where iteritems() is used avoiding the unnecessary value look-up.
    if args and isinstance(args[0], Trie):
      for key, value in args[0].iteritems():
        self[key] = value
      args = ()
    super(Trie, self).update(*args, **kwargs)

  def copy(self):
    """Returns a shallow copy of the trie."""
    return self.__class__(self)

  @classmethod
  def fromkeys(cls, keys, value=None):
    """Creates a new trie with given keys set.

    Args:
      keys: An iterable of keys that should be set in the new trie.
      value: Value to associate with given keys.

    Returns:
      A new trie where each key from ``keys`` has been set to the given value.
    """
    trie = cls()
    for key in keys:
      trie[key] = value
    return trie

  def _get_node(self, key, create=False):
    """Returns node for given key.  Creates it if requested.

    Args:
      key: A key to look for.
      create: Whether to create the node if it does not exist.

    Returns:
      ``(node, trace)`` tuple where ``node`` is the node for given key and
      ``trace`` is a list specifying path to reach the node including all the
      encountered nodes.  Each element of trace is a ``(step, node)`` tuple
      where ``step`` is a step from parent node to given node, and ``node`` is
      node on the path.  The first element of the path is always ``(None,
      self._root)``.

    Raises:
      KeyError: If there is no node for the key and ``create`` is ``False``.
    """
    node = self._root
    trace = [(None, node)]
    for step in self.__path_from_key(key):
      if create:
        node = node.children.setdefault(step, _Node())
      else:
        node = node.children.get(step)
        if not node:
          raise KeyError(key)
      trace.append((step, node))
    return node, trace

  def __iter__(self):
    return self.iterkeys()

  def iteritems(self, prefix=_SENTINEL, shallow=False):
    """Yields all nodes with associated values with given prefix.

    Args:
      prefix: Prefix to limit iteration to.
      shallow: Perform a shallow traversal, i.e. do not yield items if their
          prefix has been yielded.

    Yields:
      ``(key, value)`` tuples.

    Raises:
      KeyError: If ``prefix`` does not match any node.
    """
    node, _ = self._get_node(prefix)
    for path, value in node.iterate(list(self.__path_from_key(prefix)),
                                    shallow=shallow):
      yield (self._key_from_path(path), value)

  def iterkeys(self, prefix=_SENTINEL, shallow=False):
    """Yields all keys having associated values with given prefix.

    Args:
      prefix: Prefix to limit iteration to.
      shallow: Perform a shallow traversal, i.e. do not yield keys if their
          prefix has been yielded.

    Yields:
      All the keys (with given prefix) with associated values in the trie.

    Raises:
      KeyError: If ``prefix`` does not match any node.
    """
    for key, _ in self.iteritems(prefix=prefix, shallow=shallow):
      yield key

  def itervalues(self, prefix=_SENTINEL, shallow=False):
    """Yields all values associated with keys with given prefix.

    Args:
      prefix: Prefix to limit iteration to.
      shallow: Perform a shallow traversal, i.e. do not yield values if their
          prefix has been yielded.

    Yields:
      All the values associated with keys (with given prefix) in the trie.

    Raises:
      KeyError: If ``prefix`` does not match any node.
    """
    node, _ = self._get_node(prefix)
    for _, value in node.iterate(list(self.__path_from_key(prefix)),
                                 shallow=shallow):
      yield value

  def keys(self, prefix=_SENTINEL, shallow=False):
    """Returns a list of all the keys, with given prefix, in the trie."""
    return list(self.iterkeys(prefix=prefix, shallow=shallow))

  def values(self, prefix=_SENTINEL, shallow=False):
    """Returns a list of values in given subtrie."""
    return list(self.itervalues(prefix=prefix, shallow=shallow))

  def items(self, prefix=_SENTINEL, shallow=False):
    """Returns a list of ``(key, value)`` pairs in given subtrie."""
    return list(self.iteritems(prefix=prefix, shallow=shallow))

  def __len__(self):
    """Returns number of values in a trie.

    This method is expensive as it iterates over the whole trie.
    """
    return sum(1 for _ in self.itervalues())

  def __nonzero__(self):
    return bool(self._root)

  HAS_VALUE = 1
  HAS_SUBTRIE = 2

  def has_node(self, key):
    """Returns whether given node is in the trie.

    Args:
      key: A key to look for.

    Returns:
      A bitwise or of ``HAS_VALUE`` and ``HAS_SUBTRIE`` values indicating that
      node has a value associated with it and that it is a prefix of another
      existing key respectively.
    """
    try:
      node, _ = self._get_node(key)
    except KeyError:
      return 0
    return ((self.HAS_VALUE * int(node.value is not _SENTINEL)) |
            (self.HAS_SUBTRIE * int(bool(node.children))))

  def has_key(self, key):
    """Indicates whether given key has value associated with it."""
    return bool(self.has_node(key) & self.HAS_VALUE)

  def has_subtrie(self, key):
    """Returns whether given key is a prefix of another key in the trie."""
    return bool(self.has_node(key) & self.HAS_SUBTRIE)

  def _slice_maybe(self, key_or_slice):
    """Checks whether argument is a slice or a plain key.

    Args:
      key_or_slice: A key or a slice to test.

    Returns:
      ``(key, is_slice)`` tuple.  ``is_slice`` indicates whether
      ``key_or_slice`` is a slice and ``key`` is either ``key_or_slice`` itself
      (if it's not a slice) or slice's start position.

    Raises:
      TypeError: If ``key_or_slice`` is a slice whose stop or step are not
        ``None`` In other words, only ``[key:]`` slices are valid.
    """
    if isinstance(key_or_slice, slice):
      if key_or_slice.stop is not None or key_or_slice.step is not None:
        raise TypeError(key_or_slice)
      return key_or_slice.start, True
    return key_or_slice, False

  def __getitem__(self, key_or_slice):
    """Returns value associated with given key or raises KeyError.

    Args:
      key_or_slice: A key or a slice to look for.

    Returns:
      If a single key is passed, a value associated with given key.  If a slice
      is passed, a generator of values in specified subtrie.

    Raises:
      ShortKeyError: If the key has no value associated with it but is a prefix
          of some key with a value.  Note that :class:`ShortKeyError` is
          subclass of :class:`KeyError`.
      KeyError: If key has no value associated with it nor is a prefix of an
          existing key.
      TypeError: If ``key_or_slice`` is a slice but it's stop or step are not
          ``None``.
    """
    if self._slice_maybe(key_or_slice)[1]:
      return self.itervalues(key_or_slice.start)
    node, _ = self._get_node(key_or_slice)
    if node.value is _SENTINEL:
      raise ShortKeyError(key_or_slice)
    return node.value

  def _set(self, key, value, only_if_missing=False, clear_children=False):
    """Sets value for a given key.

    Args:
      key: Key to set value of.
      value: Value to set to.
      only_if_missing: If ``True``, value won't be changed if the key is already
          associated with a value.
      clear_children: If ``True``, all children of the node, if any, will be
          removed.

    Returns:
      Value of the node.
    """
    node, _ = self._get_node(key, create=True)
    if not only_if_missing or node.value is _SENTINEL:
      node.value = value
    if clear_children:
      node.children.clear()
    return node.value

  def __setitem__(self, key_or_slice, value):
    """Sets value associated with given key.

    Args:
      key_or_slice: A key to look for, or a slice.  If it is a slice, the whole
          subtrie (if present) will be replaced by a single node with given
          value set.
      value: Value to set.

    Raises:
      ShortKeyError: If the key has no value associated with it but is a prefix
          of some key with a value.  Note that :class:`ShortKeyError` is
          subclass of :class:`KeyError`.
      KeyError: If key has no value associated with it nor is a prefix of an
          existing key.
      TypeError: If key is a slice whose stop or step are not None.
    """
    key, is_slice = self._slice_maybe(key_or_slice)
    self._set(key, value, clear_children=is_slice)

  def setdefault(self, key, value):
    """Sets value of a given node if not set already.  Returns it afterwards."""
    return self._set(key, value, only_if_missing=True)

  def _cleanup_trace(self, trace):
    """Removes empty nodes present on specified trace.

    Args:
      trace: Trace to the node to cleanup as returned by :func:`Trie._get_node`.
    """
    i = len(trace) - 1  # len(path) >= 1 since root is always there
    step, node = trace[i]
    while i and not node:
      i -= 1
      parent_step, parent = trace[i]
      del parent.children[step]
      step, node = parent_step, parent

  def _pop_from_node(self, node, trace, default=_SENTINEL):
    """Removes a value from given node.

    Args:
      node: Node to get value of.
      trace: Trace to that node as returned by :func:`Trie._get_node`.
      default: A default value to return if node has no value set.

    Returns:
      Value of the node or ``default``.

    Raises:
      ShortKeyError: If the node has no value associated with it and ``default``
          has not been given.
    """
    if node.value is not _SENTINEL:
      value = node.value
      node.value = _SENTINEL
      self._cleanup_trace(trace)
      return value
    elif default is _SENTINEL:
      raise ShortKeyError()
    else:
      return default

  def pop(self, key, default=_SENTINEL):
    """Deletes value associated with given key and returns it.

    Args:
      key: A key to look for.  Must be iterable.
      default: If specified, value that will be returned if given key has no
          value associated with it.  If not specified, method will throw
          KeyError in such cases.

    Returns:
      Removed value, if key had value associated with it, or ``default`` (if
      given).

    Raises:
      ShortKeyError: If ``default`` has not been specified and the key has no
          value associated with it but is a prefix of some key with a value.
          Note that :class:`ShortKeyError` is subclass of :class:`KeyError`.
      KeyError: If default has not been specified and key has no value
          associated with it nor is a prefix of an existing key.
    """
    try:
      return self._pop_from_node(*self._get_node(key))
    except KeyError:
      if default is not _SENTINEL:
        return default
      raise

  def popitem(self):
    """Deletes a value from the trie.

    Returns:
      ``(key, value)`` tuple indicating deleted key.

    Raises:
      KeyError: If the trie is empty.
    """
    if not self:
      raise KeyError()
    node = self._root
    trace = [(None, node)]
    while node.value is _SENTINEL:
      step = next(node.children.iterkeys())
      node = node.children[step]
      trace.append((step, node))
    return (self._key_from_path((step for step, _ in trace[1:])),
            self._pop_from_node(node, trace))

  def __delitem__(self, key_or_slice):
    """Deletes value associated with given key or raises KeyError.

    Args:
      key_or_slice: A key to look for, or a slice.  If key is a slice, the
          whole subtrie will be removed.

    Raises:
      ShortKeyError: If the key has no value associated with it but is a prefix
          of some key with a value.  This is not thrown is key_or_slice is
          a slice -- in such cases, the whole subtrie is removed.  Note that
          :class:`ShortKeyError` is subclass of :class:`KeyError`.
      KeyError: If key has no value associated with it nor is a prefix of an
          existing key.
      TypeError: If key is a slice whose stop or step are not ``None``.
    """
    key, is_slice = self._slice_maybe(key_or_slice)
    node, trace = self._get_node(key)
    if is_slice:
      node.children.clear()
    elif node.value is _SENTINEL:
      raise ShortKeyError(key)
    node.value = _SENTINEL
    self._cleanup_trace(trace)

  def prefixes(self, key):
    """Walks towards the node specified by key and yields all found values.

    Args:
      key: Key to look for.

    Yields:
      ``(k, value)`` pairs denoting keys with associated values encountered on
      the way towards the specified key.
    """
    node = self._root
    path = self.__path_from_key(key)
    pos = 0
    while True:
      if node.value is not _SENTINEL:
        yield self._key_from_path(path[:pos]), node.value
      if pos == len(path):
        break
      node = node.children.get(path[pos])
      if not node:
        break
      pos += 1

  def shortest_prefix(self, key):
    """Finds the shortest prefix of a key with a value.

    Args:
      key: Key to look for.

    Returns:
      ``(k, value)`` where ``k`` is the shortest prefix of ``key`` (it may equal
      ``key``) and ``value`` is a value associated with that key.  If no node is
      found, ``(None, None)`` is returned.
    """
    for ret in self.prefixes(key):
      return ret
    return (None, None)

  def longest_prefix(self, key):
    """Finds the longest prefix of a key with a value.

    Args:
      key: Key to look for.

    Returns:
      ``(k, value)`` where ``k`` is the longest prefix of ``key`` (it may equal
      ``key``) and ``value`` is a value associated with that key.  If no node is
      found, ``(None, None)`` is returned.
    """
    ret = (None, None)
    for ret in self.prefixes(key):
      pass
    return ret

  def __eq__(self, other):
    return self._root == other._root  # pylint: disable=protected-access

  def __ne__(self, other):
    return self._root != other._root  # pylint: disable=protected-access

  def __str__(self):
    return 'Trie(%s)' % (
        ', '.join('%s: %s' % item for item in self.iteritems()))

  def __repr__(self):
    if self:
      return  'Trie((%s,))' % (
          ', '.join('(%r, %r)' % item for item in self.iteritems()))
    else:
      return 'Trie()'

  def __path_from_key(self, key):
    """Converts a user visible key object to internal path representation.

    Args:
      key: User supplied key or ``_SENTINEL``.

    Returns:
      An empty tuple if ``key`` was ``_SENTINEL``, otherwise whatever
      :func:`Trie._path_from_key` returns.

    Raises:
      TypeError: If ``key`` is of invalid type.
    """
    return () if key is _SENTINEL else self._path_from_key(key)

  def _path_from_key(self, key):
    """Converts a user visible key object to internal path representation.

    The default implementation simply returns key.

    Args:
      key: User supplied key.

    Returns:
      A path, which is an iterable of steps.  Each step must be hashable.

    Raises:
      TypeError: If key is of invalid type.
    """
    return key

  def _key_from_path(self, path):
    """Converts an internal path into a user visible key object.

    The default implementation creates a tuple from the path.

    Args:
      path: Internal path representation.
    Returns:
      A user visible key object.
    """
    return tuple(path)

  def traverse(self, node_factory, prefix=_SENTINEL):
    """Traverses the tree using node_factory object.

    node_factory is a callable function which accepts (path_conv, path,
    children, value=...) arguments, where path_conv is a lambda converting path
    representation to key, path is the path to this node, children is an
    iterable of children nodes constructed by node_factory, optional value is
    the value associated with the path.

    node_factory's children argument is a generator which has a few
    consequences:

    * To traverse into node's children, the generator must be iterated over.
      This can by accomplished by a simple "children = list(children)"
      statement.
    * Ignoring the argument allows node_factory to stop the traversal from going
      into the children of the node.  In other words, whole subtrie can be
      removed from traversal if node_factory chooses so.
    * If children is stored as is (i.e. as a generator) when it is iterated over
      later on it will see state of the trie as it is during the iteration and
      not when traverse method was called.

    Note: Unlike iterators, traverse method uses stack recursion which means
          that using it on deep tries may lead to a RuntimeError exception
          thrown once Python's maximum recursion depth is reached.

    Args:
      node_factory: Makes opaque objects from the keys and values of the trie.
      prefix: Prefix for node to start traversal, by default starts at root.

    Returns:
      Node object constructed by node_factory corresponding to the root node.
    """
    node, _ = self._get_node(prefix)
    return node.traverse(node_factory, self._key_from_path,
                         list(self.__path_from_key(prefix)))

class CharTrie(Trie):
  """A variant of a :class:`pygtrie.Trie` which accepts strings as keys.

  The only difference between :class:`pygtrie.CharTrie` and
  :class:`pygtrie.Trie` is that when :class:`pygtrie.CharTrie` returns keys back
  to the client (for instance in keys() method is called), those keys are
  returned as strings.
  """

  def _key_from_path(self, path):
    return ''.join(path)


class StringTrie(Trie):
  """:class:`pygtrie.Trie` variant accepting strings with a separator as keys.

  The trie accepts strings as keys which are split into components using
  a separator specified during initialisation ("/" by default).
  """

  def __init__(self, *args, **kwargs):
    """Initialises the trie.

    Except for a ``separator`` named argument, all other arguments are
    interpreted the same way :func:`Trie.update` interprets them.

    Args:
      *args: Passed to super class initialiser.
      **kwargs: Passed to super class initialiser.
      separator: A separator to use when splitting keys into paths used by the
          trie.  "/" is used if this argument is not specified.  This named
          argument is not specified on the function's prototype because of
          Python's limitations.
    """
    self._separator = kwargs.pop('separator', '/')
    super(StringTrie, self).__init__(*args, **kwargs)

  @classmethod
  def fromkeys(cls, keys, value=None, separator='/'):
    trie = cls(separator=separator)
    for key in keys:
      trie[key] = value
    return trie

  def _path_from_key(self, key):
    return key.split(self._separator)

  def _key_from_path(self, path):
    return self._separator.join(path)


class PrefixSet(_collections.MutableSet):
  """A set of prefixes.

  :class:`pygtrie.PrefixSet` works similar to a normal set except it is said to
  contain a key if the key or it's prefix is stored in the set.  For instance,
  if "foo" is added to the set, the set contains "foo" as well as "foobar".

  The set supports addition of elements but does *not* support removal of
  elements.  This is because there's no obvious consistent and intuitive
  behaviour for element deletion.
  """

  def __init__(self, iterable=None, factory=Trie, **kwargs):
    """Initialises the prefix set.

    Args:
      iterable: A sequence of keys to add to the set.
      factory: A function used to create a trie used by the
          :class:`pygtrie.PrefixSet`.
      kwargs: Additional keyword arguments passed to the factory function.
    """
    trie = factory(**kwargs)
    if iterable:
      trie.update((key, True) for key in iterable)
    self._trie = trie

  def copy(self):
    """Returns a copy of the prefix set."""
    return self.__class__(self._trie)

  def clear(self):
    """Removes all keys from the set."""
    self._trie.clear()

  def __contains__(self, key):
    """Checks whether set contains key or its prefix."""
    return bool(self._trie.shortest_prefix(key)[1])

  def __iter__(self):
    """Return iterator over all prefixes in the set.

    See :func:`PrefixSet.iter` method for more info.
    """
    return self._trie.iterkeys()

  def iter(self, prefix=_SENTINEL):
    """Iterates over all keys in the set optionally starting with a prefix.

    Since a key does not have to be explicitly added to the set to be an element
    of the set, this method does not iterate over all possible keys that the set
    contains, but only over the shortest set of prefixes of all the keys the set
    contains.

    For example, if "foo" has been added to the set, the set contains also
    "foobar", but this method will *not* iterate over "foobar".

    If ``prefix`` argument is given, method will iterate over keys with given
    prefix only.  The keys yielded from the function if prefix is given does not
    have to be a subset (in mathematical sense) of the keys yielded when there
    is not prefix.  This happens, if the set contains a prefix of the given
    prefix.

    For example, if only "foo" has been added to the set, iter method called
    with no arguments will yield "foo" only.  However, when called with
    "foobar" argument, it will yield "foobar" only.
    """
    if prefix is _SENTINEL:
      return iter(self)
    elif self._trie.has_node(prefix):
      return self._trie.iterkeys(prefix=prefix)
    elif prefix in self:
      # Make sure the type of returned keys is consistent.
      return (self._trie._key_from_path(self._trie._path_from_key(prefix)),)
    else:
      return ()

  def __len__(self):
    """Returns number of keys stored in the set.

    Since a key does not have to be explicitly added to the set to be an element
    of the set, this method does not count over all possible keys that the set
    contains (since that would be infinity), but only over the shortest set of
    prefixes of all the keys the set contains.

    For example, if "foo" has been added to the set, the set contains also
    "foobar", but this method will *not* count "foobar".
    """
    return len(self._trie)

  def add(self, key):
    """Adds given key to the set.

    If the set already contains prefix of the key being added, this operation
    has no effect.  If the key being added is a prefix of some existing keys
    in the set, those keys are deleted and replaced by a single entry for the
    key being added.

    For example, if the set contains key "foo" adding a key "foobar" does not
    change anything.  On the other hand, if the set contains keys "foobar" and
    "foobaz", adding a key "foo" will replace those two keys with a single key
    "foo".

    This makes a difference when iterating over the keys or counting number of
    keys.  Counter intuitively, adding of a key can *decrease* size of the set.

    Args:
      key: Key to add.
    """
    if key not in self:
      self._trie[key:] = True

  def discard(self, key):
    raise NotImplementedError(
      'Removing keys from PrefixSet is not implemented.')

  def remove(self, key):
    raise NotImplementedError(
      'Removing keys from PrefixSet is not implemented.')

  def pop(self):
    raise NotImplementedError(
      'Removing keys from PrefixSet is not implemented.')



#!/usr/bin/python

"""trie module unit tests."""

__author__     = 'Michal Nazarewicz <mina86@mina86.com>'
__copyright__  = 'Copyright 2014 Google Inc.'


import array
import collections
import functools
import pickle
import types
import unittest

import __init__ as pygtrie


def _UpdateTrieFactory(trie_cls, *args, **kw):
  t = trie_cls()
  t.update(*args, **kw)
  return t


def _SetterTrieFactory(trie_cls, d):
  t = trie_cls()
  for k, v in d.items():
    t[k] = v
  return t


_TRIE_FACTORIES = ((
    'TrieFromNamedArgs',
    lambda trie_cls, d: trie_cls(**d)
), (
    'TrieFromTuples',
    lambda trie_cls, d: trie_cls(d.items())
), (
    'TrieFromDict',
    lambda trie_cls, d: trie_cls(d)
), (
    'TrieFromTrie',
    lambda trie_cls, d: trie_cls(trie_cls(d))
), (
    'CopyOfTrie',
    lambda trie_cls, d: trie_cls(d).copy()
), (
    'UpdateWithNamedArgs',
    lambda trie_cls, d: _UpdateTrieFactory(trie_cls, **d)
), (
    'UpdateWithTuples',
    lambda trie_cls, d: _UpdateTrieFactory(trie_cls, d.iteritems())
), (
    'UpdateWithDict',
    _UpdateTrieFactory
), (
    'Setters',
    _SetterTrieFactory
))


class TrieTestCase(unittest.TestCase):
  # The below need to be overwritten by subclasses

  # A Trie class being tested
  _TRIE_CLS = pygtrie.Trie

  # A key to set
  _SHORT_KEY = 'foo'
  # Another key to set such that _SHORT_KEY is it's prefix
  _LONG_KEY = _SHORT_KEY + 'bar'
  # A key that is not set but _LONG_KEY is it's prefix
  _VERY_LONG_KEY = _LONG_KEY + 'baz'
  # A key that is not set and has no relation to other keys
  _OTHER_KEY = 'qux'
  # A list of prefixes of _SHORT_KEY
  _SHORT_PREFIXES = ('', 'f', 'fo')
  # A list of prefixes of _LONG_KEY which are not prefixes of _SHORT_KEY nor
  # _SHORT_KEY itself
  _LONG_PREFIXES = ('foob', 'fooba')

  def PathFromKey(self, key):
    """Turns key into a path as used by Trie class being tested."""
    return key

  def KeyFromPath(self, path):
    """Turns path as used by Trie class being tested into a key."""
    return tuple(path)

  # End of stuff that needs to be overwritten by subclasses

  def KeyFromKey(self, key):
    """Turns a key into a form that the Trie will return e.g. in keys()."""
    return self.KeyFromPath(self.PathFromKey(key))

  def assertNodeState(self, t, key, prefix=False, value=None):
    """Asserts a state of given node in a trie.

    Args:
      t: Trie to check the node in.
      key: A key for the node.
      prefix: Whether the node is a prefix of a longer key that is in the trie.
      value: If given, value associated with the key.  If missing, node has
        no value associated with it.
    Raises:
      AssertionError: If any assumption is not met.
    """
    if prefix:
      self.assertTrue(t.has_subtrie(key))
      self.assertTrue(bool(t.has_node(key) & pygtrie.Trie.HAS_SUBTRIE))
    else:
      self.assertFalse(t.has_subtrie(key))
      self.assertFalse(bool(t.has_node(key) & pygtrie.Trie.HAS_SUBTRIE))
    if value is None:
      o = object()
      self.assertNotIn(key, t)
      key_error_exception = pygtrie.ShortKeyError if prefix else KeyError
      self.assertRaises(key_error_exception, lambda: t[key])
      self.assertRaises(key_error_exception, t.pop, key)
      self.assertIsNone(t.get(key))
      self.assertIs(o, t.get(key, o))
      self.assertIs(o, t.pop(key, o))
      self.assertFalse(t.has_key(key))
      self.assertNotIn(self.KeyFromKey(key), list(t.iterkeys()))
      self.assertNotIn(self.KeyFromKey(key), t.keys())
      self.assertEquals(pygtrie.Trie.HAS_SUBTRIE if prefix else 0,
                        t.has_node(key))
    else:
      self.assertIn(key, t)
      self.assertEquals(value, t[key])
      self.assertEquals(value, t.get(key))
      self.assertEquals(value, t.get(key, object()))
      self.assertTrue(t.has_key(key))
      self.assertTrue(bool(t.has_node(key) & pygtrie.Trie.HAS_VALUE))
      self.assertIn(self.KeyFromKey(key), list(t.iterkeys()))
      self.assertIn(self.KeyFromKey(key), t.keys())

  def assertFullTrie(self, t, value=42):
    """Asserts a trie has _SHORT_KEY and _LONG_KEY set to value."""
    self.assertEquals(2, len(t))
    for prefix in self._SHORT_PREFIXES + self._LONG_PREFIXES:
      self.assertNodeState(t, prefix, prefix=True)
    self.assertNodeState(t, self._SHORT_KEY, prefix=True, value=value)
    self.assertNodeState(t, self._LONG_KEY, value=value)
    self.assertNodeState(t, self._VERY_LONG_KEY)
    self.assertNodeState(t, self._OTHER_KEY)

  def assertShortTrie(self, t, value=42):
    """Asserts a trie has only _SHORT_KEY set to value."""
    self.assertEquals(1, len(t))
    for prefix in self._SHORT_PREFIXES:
      self.assertNodeState(t, prefix, prefix=True)
    for key in self._LONG_PREFIXES + (
        self._LONG_KEY, self._VERY_LONG_KEY, self._OTHER_KEY):
      self.assertNodeState(t, key)
    self.assertNodeState(t, self._SHORT_KEY, value=value)

  def assertEmptyTrie(self, t):
    """Asserts a trie is empty."""
    self.assertEquals(0, len(t), '%r should be empty: %d' % (t, len(t)))

    for key in self._SHORT_PREFIXES + self._LONG_PREFIXES + (
        self._SHORT_KEY, self._LONG_KEY, self._VERY_LONG_KEY,
        self._OTHER_KEY):
      self.assertNodeState(t, key)

    self.assertRaises(KeyError, t.popitem)

    self.assertEquals('Trie()', str(t))
    self.assertEquals('Trie()', repr(t))

  def DoTestBasics(self, trie_factory):
    """Basic trie tests."""
    d = dict.fromkeys((self._SHORT_KEY, self._LONG_KEY), 42)
    t = trie_factory(self._TRIE_CLS, d)

    self.assertFullTrie(t)

    self.assertEquals(42, t.pop(self._LONG_KEY))
    self.assertShortTrie(t)

    self.assertEquals(42, t.setdefault(self._SHORT_KEY, 24))
    self.assertShortTrie(t)

    t[self._SHORT_KEY] = 24
    self.assertShortTrie(t, 24)

    self.assertEquals(24, t.setdefault(self._LONG_KEY, 24))
    self.assertFullTrie(t, 24)

    del t[self._LONG_KEY]
    self.assertShortTrie(t, 24)

    self.assertEquals((self.KeyFromKey(self._SHORT_KEY), 24), t.popitem())
    self.assertEmptyTrie(t)

  def DoTestIterator(self, trie_factory):
    """Trie iterator tests."""
    d = dict.fromkeys((self._SHORT_KEY, self._LONG_KEY), 42)
    t = trie_factory(self._TRIE_CLS, d)

    self.assertEquals([42, 42], t.values())
    self.assertEquals([42, 42], list(t.itervalues()))

    short_key = self.KeyFromKey(self._SHORT_KEY)
    long_key = self.KeyFromKey(self._LONG_KEY)

    expected_items = [(short_key, 42), (long_key, 42)]
    self.assertEquals(expected_items, sorted(t.items()))
    self.assertEquals(expected_items, sorted(t.iteritems()))

    self.assertEquals([short_key, long_key], sorted(t))
    self.assertEquals([short_key, long_key], sorted(t.keys()))
    self.assertEquals([short_key, long_key], sorted(t.iterkeys()))

  def DoTestSubtrieIterator(self, trie_factory):
    """Subtrie iterator tests."""
    d = dict.fromkeys((self._SHORT_KEY, self._LONG_KEY), 42)
    t = trie_factory(self._TRIE_CLS, d)

    long_key = self.KeyFromKey(self._LONG_KEY)
    prefix = self._LONG_PREFIXES[0]

    self.assertEquals([42, 42], t.values(prefix=self._SHORT_KEY))
    self.assertEquals([42, 42], list(t.itervalues(prefix=self._SHORT_KEY)))
    self.assertEquals([42], t.values(prefix=prefix))
    self.assertEquals([42], list(t.itervalues(prefix=prefix)))

    expected_items = [(long_key, 42)]
    self.assertEquals(expected_items, sorted(t.items(prefix=prefix)))
    self.assertEquals(expected_items, sorted(t.iteritems(prefix=prefix)))

    self.assertEquals([long_key], sorted(t.keys(prefix=prefix)))
    self.assertEquals([long_key], sorted(t.iterkeys(prefix=prefix)))

  def DoTestShallowIterator(self, trie_factory):
    """Shallow iterator test."""
    d = dict.fromkeys((self._SHORT_KEY, self._LONG_KEY), 42)
    t = trie_factory(self._TRIE_CLS, d)

    self.assertEquals([42], t.values(shallow=True))
    self.assertEquals([42], list(t.itervalues(shallow=True)))

    short_key = self.KeyFromKey(self._SHORT_KEY)
    long_key = self.KeyFromKey(self._LONG_KEY)

    expected_items = [(short_key, 42)]
    self.assertEquals(expected_items, sorted(t.items(shallow=True)))
    self.assertEquals(expected_items, sorted(t.iteritems(shallow=True)))

    self.assertEquals([short_key], sorted(t.keys(shallow=True)))
    self.assertEquals([short_key], sorted(t.iterkeys(shallow=True)))

  def DoTestSpliceOperations(self, trie_factory):
    """Splice trie operations tests."""
    d = dict.fromkeys((self._SHORT_KEY, self._LONG_KEY), 42)
    t = trie_factory(self._TRIE_CLS, d)

    self.assertEquals([42, 42], list(t[self._SHORT_KEY:]))
    self.assertEquals([42], list(t[self._LONG_PREFIXES[0]:]))

    t[self._SHORT_KEY:] = 24
    self.assertShortTrie(t, 24)

    self.assertEquals([24], list(t[self._SHORT_KEY:]))
    self.assertRaises(KeyError, lambda: list(t[self._LONG_PREFIXES[0]:]))

    t[self._LONG_KEY:] = 24
    self.assertFullTrie(t, 24)

    del t[self._SHORT_KEY:]
    self.assertEmptyTrie(t)

  def DoTestFindPrefix(self, trie_factory):
    """Prefix finding methods tests."""
    d = dict.fromkeys((self._SHORT_KEY, self._LONG_KEY), 42)
    t = trie_factory(self._TRIE_CLS, d)

    short_pair = (self.KeyFromKey(self._SHORT_KEY), 42)
    long_pair = (self.KeyFromKey(self._LONG_KEY), 42)
    none_pair = (None, None)

    self.assertEquals(short_pair, t.shortest_prefix(self._VERY_LONG_KEY))
    self.assertEquals(short_pair, t.shortest_prefix(self._LONG_KEY))
    self.assertEquals(short_pair, t.shortest_prefix(self._VERY_LONG_KEY))
    self.assertEquals(short_pair, t.shortest_prefix(self._LONG_PREFIXES[-1]))
    self.assertEquals(short_pair, t.shortest_prefix(self._SHORT_KEY))
    self.assertEquals(none_pair, t.shortest_prefix(self._SHORT_PREFIXES[-1]))

    self.assertEquals(long_pair, t.longest_prefix(self._VERY_LONG_KEY))
    self.assertEquals(long_pair, t.longest_prefix(self._LONG_KEY))
    self.assertEquals(long_pair, t.longest_prefix(self._VERY_LONG_KEY))
    self.assertEquals(short_pair, t.shortest_prefix(self._LONG_PREFIXES[-1]))
    self.assertEquals(short_pair, t.longest_prefix(self._SHORT_KEY))
    self.assertEquals(none_pair, t.shortest_prefix(self._SHORT_PREFIXES[-1]))

    self.assertEquals([], list(t.prefixes(self._SHORT_PREFIXES[-1])))
    self.assertEquals([short_pair], list(t.prefixes(self._SHORT_KEY)))
    self.assertEquals([short_pair], list(t.prefixes(self._LONG_PREFIXES[-1])))
    self.assertEquals([short_pair, long_pair], list(t.prefixes(self._LONG_KEY)))
    self.assertEquals([short_pair, long_pair],
                      list(t.prefixes(self._VERY_LONG_KEY)))

  def DoTestPickle(self, trie_factory):
    """https://github.com/google/pygtrie/issues/7"""
    d = dict.fromkeys((self._SHORT_KEY, self._LONG_KEY, self._VERY_LONG_KEY,
                       self._OTHER_KEY), 42)
    t = trie_factory(self._TRIE_CLS, d)

    pickled = pickle.dumps(t)
    u = pickle.loads(pickled)

    self.assertEquals(t, u)

  def testPrefixSet(self):
    """PrefixSet test."""
    ps = pygtrie.PrefixSet(factory=self._TRIE_CLS)

    short_key = self.KeyFromKey(self._SHORT_KEY)
    long_key = self.KeyFromKey(self._LONG_KEY)
    very_long_key = self.KeyFromKey(self._VERY_LONG_KEY)
    other_key = self.KeyFromKey(self._OTHER_KEY)

    for key in (self._LONG_KEY, self._VERY_LONG_KEY):
      ps.add(key)
      self.assertEquals(1, len(ps))
      self.assertEquals([long_key], list(ps.iter()))
      self.assertEquals([long_key], list(iter(ps)))
      self.assertEquals([long_key], list(ps.iter(self._SHORT_KEY)))
      self.assertEquals([long_key], list(ps.iter(self._LONG_KEY)))
      self.assertEquals([very_long_key], list(ps.iter(self._VERY_LONG_KEY)))
      self.assertEquals([], list(ps.iter(self._OTHER_KEY)))

    ps.add(self._SHORT_KEY)
    self.assertEquals(1, len(ps))
    self.assertEquals([short_key], list(ps.iter()))
    self.assertEquals([short_key], list(iter(ps)))
    self.assertEquals([short_key], list(ps.iter(self._SHORT_KEY)))
    self.assertEquals([long_key], list(ps.iter(self._LONG_KEY)))
    self.assertEquals([], list(ps.iter(self._OTHER_KEY)))

    ps.add(self._OTHER_KEY)
    self.assertEquals(2, len(ps))
    self.assertEquals(sorted((short_key, other_key)),
                      list(ps.iter()))
    self.assertEquals([short_key], list(ps.iter(self._SHORT_KEY)))
    self.assertEquals([long_key], list(ps.iter(self._LONG_KEY)))
    self.assertEquals([other_key], list(ps.iter(self._OTHER_KEY)))

  def testEquality(self):
    """Tests equality comparison."""
    d = dict.fromkeys((self._SHORT_KEY, self._LONG_KEY), 42)
    # pylint: disable=redefined-outer-name
    tries = [factory(self._TRIE_CLS, d) for _, factory in _TRIE_FACTORIES]

    for i in range(1, len(tries)):
      self.assertEquals(tries[i-1], tries[i],
                        '%r (factory: %s) should equal %r (factory: %s)' %
                        (tries[i-1], _TRIE_FACTORIES[i-1][0],
                         tries[i], _TRIE_FACTORIES[i][0]))

    for i in range(1, len(tries)):
      tries[i-1][self._OTHER_KEY] = 42
      self.assertNotEquals(
          tries[i-1], tries[i],
          '%r (factory: %s) should not be equal %r (factory: %s)' %
          (tries[i-1], _TRIE_FACTORIES[i-1][0],
           tries[i], _TRIE_FACTORIES[i][0]))


for method_name in TrieTestCase.__dict__.keys():  # pylint: disable=g-builtin-op
  if method_name.startswith('DoTest'):
    original_method = getattr(TrieTestCase, method_name)
    method_name = 't' + method_name[3:]
    for factory_name, factory in _TRIE_FACTORIES:
      method = functools.partial(original_method, trie_factory=factory)
      method.__doc__ = '%s using %s trie factory.' % (
          original_method.__doc__[:-2], factory_name)
      method = types.MethodType(method, None, TrieTestCase)
      setattr(TrieTestCase, '%s_%s' % (method_name, factory_name), method)


class CharTrieTestCase(TrieTestCase):
  _TRIE_CLS = pygtrie.CharTrie

  def KeyFromPath(self, path):
    return ''.join(path)


class StringTrieTestCase(TrieTestCase):
  _TRIE_CLS = pygtrie.StringTrie

  _SHORT_KEY = '/home/foo'
  _LONG_KEY = _SHORT_KEY + '/bar/baz'
  _VERY_LONG_KEY = _LONG_KEY + '/qux'
  _OTHER_KEY = '/hom'
  _SHORT_PREFIXES = ('', '/home')
  _LONG_PREFIXES = ('/home/foo/bar',)

  def PathFromKey(self, key):
    return key.split('/')

  def KeyFromPath(self, path):
    return '/'.join(path)



class TraverseTest(unittest.TestCase):

  _SENTINEL = object()
  _TestNode = collections.namedtuple('TestNode', 'key children value')

  @classmethod
  def _make_test_node(cls, path_conv, path, children, value=_SENTINEL):
    return cls._TestNode(path_conv(path), list(children), value)

  def assertNode(self, node, key, children=0, value=_SENTINEL):
    self.assertTrue(node)
    self.assertEquals(key, node.key)
    self.assertEquals(children, len(node.children))
    self.assertEquals(value, node.value)
    return node

  def testTraverseEmptyTree(self):
    t = pygtrie.CharTrie()
    r = t.traverse(self._make_test_node)
    self.assertNode(r, '', 0)

  def testTraverseSingletonTree(self):
    t = pygtrie.CharTrie()
    t.update({'a': 10})

    r = t.traverse(self._make_test_node)
    self.assertNode(r, '', 1)
    self.assertNode(r.children[0], 'a', 0, 10)

  def testTraverse(self):
    t = pygtrie.CharTrie()
    t.update({'aaa': 1, 'aab': 2, 'aac': 3, 'bb': 4})

    r = t.traverse(self._make_test_node)
    # Result:
    #  <>
    #    a
    #      aa
    #        aaa:1
    #        aab:2
    #        aac:3
    #    b
    #      bb:4
    self.assertNode(r, '', 2)

    a_node = self.assertNode(r.children[0], 'a', 1)
    aa_node = self.assertNode(a_node.children[0], 'aa', 3)
    self.assertNode(aa_node.children[0], 'aaa', 0, 1)
    self.assertNode(aa_node.children[2], 'aac', 0, 3)

    b_node = self.assertNode(r.children[1], 'b', 1)
    self.assertNode(b_node.children[0], 'bb', 0, 4)

  def testTraverseCompressing(self):
    t = pygtrie.CharTrie()
    t.update({'aaa': 1, 'aab': 2, 'aac': 3, 'bb': 4})

    def make(path_conv, path, children, value=self._SENTINEL):
      children = list(children)
      if value is self._SENTINEL and len(children) == 1:
        # There is only one prefix.
        return children[0]
      else:
        return self._TestNode(path_conv(path), children, value)

    r = t.traverse(make)
    # Result:
    # <>
    #  aa
    #    aaa:1
    #    aab:2
    #    aac:3
    #  bb:4
    self.assertNode(r, '', 2)

    aa_node = self.assertNode(r.children[0], 'aa', 3)
    self.assertNode(aa_node.children[0], 'aaa', 0, 1)
    self.assertNode(aa_node.children[1], 'aab', 0, 2)
    self.assertNode(aa_node.children[2], 'aac', 0, 3)

    self.assertNode(r.children[1], 'bb', 0, 4)

  def testTraverseIgnoreSubtrie(self):
    t = pygtrie.CharTrie()
    t.update({'aaa': 1, 'aab': 2, 'aac': 3, 'b': 4})

    cnt = [0]

    def make(path_conv, path, children, value=self._SENTINEL):
      cnt[0] += 1
      if path and path[0] == 'a':
        return None
      else:
        return self._TestNode(path_conv(path), filter(None, children), value)

    r = t.traverse(make)
    # Result:
    # <>
    #  b:4
    self.assertNode(r, '', 1)
    self.assertNode(r.children[0], 'b', 0, 4)
    self.assertEquals(3, cnt[0])


class RecursionTest(unittest.TestCase):
  """Test for deep recursion.

  https://github.com/google/pygtrie/issues/8
  """

  def CreateTrie(self):
    trie = pygtrie.Trie()
    for x in range(100):
      y = array.array('h', range(x, 1000)).tostring()
      trie.update([(y,x,)])
    return trie

  def testIterator(self):
    for _ in self.CreateTrie().iteritems():
      pass

  def testCopy(self):
    self.CreateTrie().copy()


if __name__ == '__main__':
  unittest.main()
